@@ -45,7 +45,8 @@ setClass(
 		manipKeyVars='dataframeOrNULL',
 		manipNumVars='dataframeOrNULL',
 		manipStrataVar='factorOrNULL',
-		risk='listOrNULL',
+    originalRisk='listOrNULL',
+    risk='listOrNULL',
 		utility='listOrNULL',
 		pram='listOrNULL',
 		localSuppression='listOrNULL',
@@ -64,7 +65,8 @@ setClass(
 		manipKeyVars=NULL,
 		manipNumVars=NULL,
 		manipStrataVar=NULL,
-		risk=NULL,
+    originalRisk=NULL,
+    risk=NULL,
 		utility=NULL,
 		pram=NULL,
 		localSuppression=NULL,
